package cl.asa.parse.cabocha
/**
 *   cabochaの解析結果より文節間の関係や，語義付与に必要な動詞や格助詞などの情報の付与するクラス
 */
import cl.asa.result._

class Basic {

	def parse(result: Result): Result = {
		result.chunks.foreach { chunk =>
			chunk.surface = getChunkSurface(chunk)
			chunk.modifyingchunk = getModifyingChunk(result, chunk)
			chunk.modifiedchunks = getModifiedChunks(result, chunk)
			chunk.ctype = getChunkType(chunk)
			chunk.main = getHead(chunk)
			chunk.part = getPart(chunk)
			chunk.morphs.foreach(_.chunk = chunk)
		}
		return result
	}

	/**
	 * 文節内の形態素の表層をつなげて，文節の表層を取得
	 * @param chunk 文節
	 * @return 文節の表層
	 */
	private def getChunkSurface(chunk: Chunk): String = {
		val surface: String = chunk.morphs.map(_.surface).mkString
		return surface
	}

	/**
	 * 係っている文節を取得
	 */
	private def getModifyingChunk(result: Result, chunk: Chunk): Chunk = {
		val modifyingchunk: Chunk = chunk.link match {
			case -1 => null
			case _ => result.chunks(chunk.link)
		}
		return modifyingchunk
	}

	/**
	 * 文節の係りを受けている文節を取得
	 * @param chunk 文節
	 * @return 係り元の文節集合
	 */
	private def getModifiedChunks(result: Result, depchunk: Chunk): Seq[Chunk] = {
		val linkchunks: Seq[Chunk] = result.chunks.filter { chunk =>
			chunk.link.equals(depchunk.id)
		}
		return linkchunks
	}

	/**
	 * 文節のタイプを取得
	 *  - verb:      動詞
	 *  - adjective: 形容詞，形容動詞
	 *  - copula:    コピュラ(AはBだ)
	 *  - elem:      その他
	 */
	private def getChunkType(chunk: Chunk): String = {
		var ctype: String = chunk.morphs match {
			case morphs if morphs.exists { morph =>
				morph.pos.contains("動詞,自立")
			} => "verb"
			case morphs if morphs.exists { morph =>
				morph.pos.contains("形容詞,自立") | morph.pos.contains("形容動詞語幹")
			} => "adjective"
			case morphs if morphs.exists { morph =>
				morph.cform.contains("特殊・ダ") | morph.cform.contains("特殊・デス")
			} => "copula"
			case _ => "elem"
		}
		return ctype
	}

	/**
	 * その文節の主辞となるような語の取得(意味役割付与に使用)
	 */
	private def getHead(chunk: Chunk): String = {
		val head: String = chunk.ctype match {
			case "copula" =>
				chunk.morphs.withFilter { morph =>
					morph.pos.contains("名詞") || morph.cform.equals("特殊・ダ") || morph.cform.equals("特殊・デス")
				}.map(_.base).mkString
			case "verb" =>
				chunk.morphs.filter { morph =>
					morph.pos.contains("サ変接続") | morph.pos.contains("動詞,自立")
				} match {
					case morphs if morphs.size == 1 => morphs.head match {
						case morph if morph.base.equals("する") =>
							chunk.morphs.find(_.id.equals(morph.id - 1)) match {
								case Some(premorph) => premorph.surface + morph.base
								case None => morph.base
							}
						case morph if morph.pos.contains("動詞,自立") => morph.base
					}
					case morphs if morphs.size > 1 =>
						morphs.init.map(_.surface).mkString + morphs.last.base
				}
			case "adjective" =>
				chunk.morphs.filter { morph =>
					morph.pos.contains("形容詞,自立") | morph.pos.contains("形容動詞語幹")
				}.head match {
					case morph if morph.pos.contains("形容詞,自立") => morph.base
					case morph if morph.pos.contains("形容動詞語幹") => morph.base + "だ"
				}
			case "elem" =>
				chunk.morphs.withFilter {morph => 
					morph.pos.contains("名詞") || morph.pos.contains("副詞")
				}.map(_.surface).mkString
		}
		return head
	}

	/**
	 * 文節内の名詞につく格助詞の取得
	 * @param chunk 文節
	 * @return 文節内の格助詞or係助詞
	 */
	private def getPart(chunk: Chunk): String = {
		
		val part: String = chunk.morphs.filter { morph =>
			morph.pos.contains("格助詞") | morph.pos.contains("係助詞")
		}.lastOption match {
			case Some(morph) => morph.base
			case None => ""
		}
		return part
	}
}